(*=====================================================================*)
(*                                                                     *)
(*          Run-time Code Generation Library for Objective Caml        *)
(*                                                                     *)
(*                   Dmitri Lomov (dsl@tepkom.ru)                      *)
(*              St-Petersburg State University, Russia                 *)
(*                                                                     *)
(*=====================================================================*)    

(* $Id: rtcg.mli,v 1.2 2002/03/18 09:14:40 mitya Exp $ *)

(* rtcg.mli *)

(** Module [Rtcg] contains primitives for code creation.**)

type 'a code
(** Basic abstract type. Semantics is the following: value of
    type ['a code], being evaluated, produces a value of type ['a].
**)
    

(*+ Core functions +*)

val abstr : ('a code -> 'b code) -> ('a -> 'b) code
(** Function abstraction. ``Raises'' a function over [code] 
    domain. **) 

val apply : ('a -> 'b) code -> 'a code -> 'b code
(** Creates code of function application. **)
val quote : 'a -> 'a code
(** Quotes arbitrary OCaml value in code. **)
val fix   : (('a -> 'b) code -> ('a -> 'b) code) -> 
  ('a -> 'b) code
(** Fixpoint operator for recursive functions. **)
val let'  : 'a code -> ('a code -> 'b code) -> 'b code
(** Functional representation of a [let] construct. **)

(*+ Booleans and conditionals +*)
val if' : bool code -> 'a code -> 'a code -> 'a code
(** Equivalent for [if]-statement. Creates code that evaluates either 
    second or third argument depending on whether result of
    evaluating first argument is [true] or [false] **) 

val bor : bool code -> bool code -> bool code  
(** Boolean ``or''. **)
val band : bool code -> bool code -> bool code 
(** Boolean ``and''. **)
val bnot : bool code -> bool code
(** Boolean ``not''. **)
(** All boolean operations have McCarthy semantics. **)

(*+ Equality +*)
val eq : 'a code -> 'a code -> bool code  
(** Test for physical equality (like OCaml [==]). **)
val neq : 'a code -> 'a code -> bool code 
(** Test for physical inequality (like OCaml [!=]). **)
val equal : 'a code -> 'a code -> bool code
(** Test for structural equality (like OCaml [=]). **)

type comparison = LT | GT | LE | GE | EQ | NEQ
(** Comparison type **)

val compare : 'a code -> comparison -> 'a code -> bool code
(** Structural ordering of values **)

(*+ Tuples +*)
val tuple2 : 'a1 code -> 'a2 code -> ('a1*'a2) code
val tuple3 : 'a1 code -> 'a2 code -> 'a3 code -> 
  ('a1*'a2*'a3) code
(** Functions for creating tuples of length 2 and 3. **)

val slice20 : ('a*'b) code -> 'a code
val slice21 : ('a*'b) code -> 'b code
val slice30 : ('a*'b*'c) code -> 'a code 
val slice31 : ('a*'b*'c) code -> 'b code 
val slice32 : ('a*'b*'c) code -> 'c code 
(** Accessing tuple elements --- tuples of length 2 and 3. **)

val unsafe_tuple : Obj.t -> Obj.t 
(** Unsafe function for creating code creating tuples. \\
    An argument must be OCaml memory block containing codes of
    all elements of a tuple. Result is code creating tuple with
    that elements. \\
    This function should never be used directly. **)
val unsafe_slice : Obj.t -> int -> Obj.t
(** Unsafe function for creating code for getting elements of
    tuples. Arguments are tuple code and element to get (first
    has index 0). Result is a code for obtaining an element of 
    a tuple. \\ 
    This function should never be used directly. **)

(*+ Imperative +*)
val seq : unit code -> 'a code -> 'a code
(** Create [code] that sequence evaluation of two [code]s. **)
val lose : 'a code -> unit code
(** ``Loses'' result of [code] so that it can be sequenced. **)
type direction = DTo | DDownTo
(** Direction of [for] loop. **)
val for' : int code -> direction -> int code -> 
  (int code -> unit code) -> unit code
(** For loop. 3rd argument is applied for values starting 
    from value of first argument, to the value of third 
    argument increasing or decreasing depending on second 
    argument. **)
val while' : bool code -> unit code -> unit code
(** While loop. **)

(*+ Arrays +*)
val array_get : 'a array code -> int code -> 'a code
(** Creates [code] to get an element from array **)
val array_set : 'a array code -> int code -> 'a code -> 
  unit code
(** Creates [code] to set an element of an array **)
val array_length : 'a array code -> int code
(** Creates [code] that calculates array length **)

(*+ Strings +*)
val string_get : string code -> int code -> char code
(** Creates [code] to get a character from a string **)
val string_set : string code -> int code -> char code -> 
  unit code
(** Creates [code] to set a character of a string **)

(*+ References +*)
val ref_create : 'a code -> 'a ref code
(** Creates [code] of a reference **)

val ref_value : 'a ref code -> 'a code
(** Gets value of reference **)

val ref_set : 'a ref code -> 'a code -> unit code
(** Assign a value to a reference **)

(*+ Integers +*)
type int_bin_op = 
    IOpSUB | IOpADD | IOpMUL | IOpDIV | IOpMOD | 
    IOpAND | IOpOR  | IOpXOR |
    IOpLSL | IOpLSR | IOpASR 
(** Binary operations with integers **)

type int_un_op = IUnNEG | IUnNOT
(** Unary operations with integers **)


val int_bin : int code -> int_bin_op -> int code -> int code
(** Code of binary integer operation **)
val int_un : int_un_op -> int code -> int code
(** Code of unary integer operation **)
val int_cmp : int code -> comparison -> int code -> bool code
(** Code of integer comparison **)

(*+ Floats +*)

type flop = 
    FLOP_ADD | FLOP_SUB | FLOP_MUL | FLOP_DIV | 
    FLOP_MOD | FLOP_POWER
(** Floating-point operations **)

val float_bin : float code -> flop -> float code -> float code
(** Code of floating-point binary operation **)
val float_neg : float code -> float code
(** Code of floating-point negation **)









